@@ -1,5 +1,8 @@
 use std::{
-    collections::{hash_map::Entry::Vacant, HashMap, HashSet},
+    collections::{
+        hash_map::Entry::{self, Vacant},
+        HashMap, HashSet,
+    },
     iter::zip,
     rc::Rc,
 };
@@ -10,9 +13,8 @@ use itertools::Itertools;
 use crate::{pyo3_prelude::*, util::BitMask128};
 
 use super::{
-    computational_node::EinsumArgs, deep_map_unwrap, evaluate_fn, Add, Circuit,
-    CircuitConstructionError, CircuitNode, CircuitRc, Concat, Einsum, GeneralFunction, HashBytes,
-    Index, Rearrange, ScalarConstant, Scatter, Symbol,
+    computational_node::EinsumArgs, deep_map_unwrap, evaluate_fn, prelude::*, Add, Concat, Einsum,
+    GeneralFunction, HashBytes, Index, Rearrange, ScalarConstant, Scatter, Symbol,
 };
 use crate::{
     filter_by_variant,
@@ -1530,3 +1532,26 @@ pub fn add_outer_product_broadcasts_on_top(add: &Add) -> Option<Add> {
         .collect();
     Some(Add::try_new(new_operands, None).unwrap())
 }
+
+/// returns None if sub isn't subset
+#[pyfunction]
+pub fn extract_add(add: &Add, sub: &Add) -> Option<Add> {
+    let mut counts = add.to_counts();
+
+    for item in &sub.nodes {
+        match counts.entry(item.clone()) {
+            Entry::Occupied(mut entry) => {
+                let new = entry.get().saturating_sub(1);
+                if new == 0 {
+                    entry.remove_entry();
+                } else {
+                    *entry.get_mut() = new;
+                }
+            }
+            Entry::Vacant(_) => return None,
+        }
+    }
+    *counts.entry(sub.clone().rc()).or_insert(0) += 1;
+
+    Some(Add::try_from_counts(&counts, add.name_cloned()).unwrap())
+}
