@@ -7,18 +7,18 @@ use crate::{circuit::print::rust_expression_notation_circuit, pyo3_prelude::*};
 use super::{
     algebraic_rewrite::deep_optimize_einsums,
     canonicalize::deep_canonicalize,
-    circuit_utils::cast_circuit,
     compiler_heuristics::deep_maybe_distribute,
     compiler_strip::strip_names,
     debugging::{create_circuit_test, randn_ablate_great_grandchildren},
     deep_rewrite::deep_heuristic_nest_adds,
     deep_rewrite::{compiler_simp, deep_push_down_index},
-    flat_concat, get_compatible_dtype,
+    flat_concat,
+    prelude::*,
     scheduled_execution::evaluate_schedule_adjust_numerical_scale,
     scheduled_execution::{
         circuit_to_schedule, circuit_to_schedule_naive_toposort, evaluate_schedule, SchedulingError,
     },
-    Circuit, CircuitNode, CircuitRc, HashBytes,
+    CircuitNode, CircuitRc, HashBytes,
 };
 use crate::{
     py_types::{Tensor, UN_FLAT_CONCAT},
