@@ -15,8 +15,8 @@ use std::{
 
 use super::{
     algebraic_rewrite::{add_deduplicate, add_flatten_once, remove_add_few_input},
-    CachedCircuitInfo, Circuit, CircuitConstructionError, CircuitNode, CircuitNodeAutoName,
-    CircuitRc, PyCircuitBase, ScalarConstant, Shape,
+    prelude::*,
+    CachedCircuitInfo, CircuitConstructionError, PyCircuitBase, ScalarConstant, Shape,
 };
 use crate::{
     circuit_node_auto_impl,
@@ -427,7 +427,6 @@ pub struct Add {
 
 impl Add {
     #[apply(new_rc_unwrap)]
-
     pub fn try_new(
         nodes: Vec<CircuitRc>,
         name: Option<String>,
@@ -445,6 +444,19 @@ impl Add {
         out.init_info()
     }
 
+    pub fn try_from_counts(
+        nodes: &HashMap<CircuitRc, usize>,
+        name: Option<String>,
+    ) -> Result<Self, CircuitConstructionError> {
+        Self::try_new(
+            nodes
+                .iter()
+                .flat_map(|(circ, &count)| vec![circ.clone(); count])
+                .collect(),
+            name,
+        )
+    }
+
     fn compute_shape_maybe(&self) -> Result<Shape, TensorInvariantError> {
         let shape = broadcast_shapes(&self.nodes.iter().map(|x| x.info().shape.clone()).collect());
         shape
@@ -486,6 +498,15 @@ impl Add {
     pub fn elim_few(&self) -> CircuitRc {
         remove_add_few_input(self).unwrap_or(self.clone().rc())
     }
+
+    pub fn to_counts(&self) -> HashMap<CircuitRc, usize> {
+        let mut counts = HashMap::new();
+        for item in &self.nodes {
+            *counts.entry(item.clone()).or_insert(0) += 1;
+        }
+
+        counts
+    }
 }
 
 circuit_node_extra_impl!(Add);
