@@ -6,8 +6,8 @@ use std::collections::HashMap;
 use uuid::Uuid;
 
 use super::{
-    named_axes::set_named_axes, CachedCircuitInfo, Circuit, CircuitConstructionError, CircuitNode,
-    CircuitRc, PyCircuitBase, Shape, TensorEvalError,
+    named_axes::set_named_axes, prelude::*, CachedCircuitInfo, PyCircuitBase, Shape,
+    TensorEvalError,
 };
 use crate::{
     circuit_node_auto_impl, circuit_node_extra_impl,
@@ -60,10 +60,6 @@ impl CircuitNode for ArrayConstant {
         self.value.shape().clone()
     }
 
-    fn max_non_input_size(&self) -> BigUint {
-        BigUint::from(0usize)
-    }
-
     fn compute_hash(&self) -> blake3::Hasher {
         let mut hasher = blake3::Hasher::new();
         hasher.update(self.value.hash().unwrap());
