@@ -9,8 +9,7 @@ use crate::{
 };
 
 use super::{
-    algebraic_rewrite::add_make_broadcasts_explicit, Circuit, CircuitNode, Concat, Einsum, Index,
-    Rearrange,
+    algebraic_rewrite::add_make_broadcasts_explicit, prelude::*, Concat, Einsum, Index, Rearrange,
 };
 
 use crate::pyo3_prelude::*;
