@@ -66,13 +66,17 @@ use self::{
 
 pub type NamedAxes = Vec<Option<String>>;
 pub trait CircuitNode: CircuitNodePrivate {
+    // ==== implementable section ===
+    //
+    // NOTE: ALL FNS IN THIS SECTION *MUST* BE COPIED TO THE CIRCUIT NODE UNION IMPL!
+    // If you add something here with a default impl, write a new impl for circuit node union!
+    // (up until default section)
+    //
+    // we could enforce this sort of stuff with some proc macros, but seems like overkill atm.
+
     fn info(&self) -> &CachedCircuitInfo;
     fn name(&self) -> Option<&str>;
 
-    fn name_cloned(&self) -> Option<String> {
-        self.name().map(|x| x.to_owned())
-    }
-
     fn compute_shape(&self) -> Shape;
     fn compute_hash(&self) -> blake3::Hasher; // shouldn't hash name
     fn compute_is_constant(&self) -> bool {
@@ -89,6 +93,36 @@ pub trait CircuitNode: CircuitNodePrivate {
         Box::new(std::iter::empty())
     }
 
+    fn child_axis_map(&self) -> Vec<Vec<Option<usize>>>;
+
+    fn children<'a>(&'a self) -> Box<dyn Iterator<Item = CircuitRc> + 'a>;
+
+    fn map_children_enumerate<'a, F, E>(&'a self, f: F) -> Result<Self, CircuitConstructionError>
+    where
+        Self: Sized,
+        CircuitConstructionError: From<E>,
+        F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>;
+
+    fn node_type_uuid(&self) -> [u8; 16];
+
+    fn self_flops(&self) -> BigUint {
+        BigUint::from(0usize)
+    }
+
+    fn eval_tensors(
+        &self,
+        tensors: &[Tensor],
+        device_dtype: &TorchDeviceDtype,
+    ) -> Result<Tensor, TensorEvalError>;
+
+    // ==== default section ===
+    // FUNCTIONS BELOW HERE *shouldn't* be overridden by implementors!
+    // (if you do implement, this won't be picked up on by union types!)
+
+    fn name_cloned(&self) -> Option<String> {
+        self.name().map(|x| x.to_owned())
+    }
+
     fn compute_device_dtype(&self) -> Result<TorchDeviceDtypeOp, CircuitConstructionError> {
         self.children()
             .map(|c| c.info().device_dtype.clone())
@@ -98,8 +132,6 @@ pub trait CircuitNode: CircuitNodePrivate {
             })
     }
 
-    fn child_axis_map(&self) -> Vec<Vec<Option<usize>>>;
-
     fn compute_named_axes(&self) -> NamedAxes {
         let child_axis_map = self.child_axis_map();
         let mut result: Vec<Option<String>> = vec![None; self.info().rank()];
@@ -113,14 +145,6 @@ pub trait CircuitNode: CircuitNodePrivate {
         result
     }
 
-    fn children<'a>(&'a self) -> Box<dyn Iterator<Item = CircuitRc> + 'a>;
-
-    fn map_children_enumerate<'a, F, E>(&'a self, f: F) -> Result<Self, CircuitConstructionError>
-    where
-        Self: Sized,
-        CircuitConstructionError: From<E>,
-        F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>;
-
     fn map_children<'a, F, E>(&'a self, mut f: F) -> Result<Self, CircuitConstructionError>
     where
         Self: Sized,
@@ -187,7 +211,7 @@ pub trait CircuitNode: CircuitNodePrivate {
             None
         }
     }
-    fn node_type_uuid(&self) -> [u8; 16];
+
     fn init_info(mut self) -> Result<Self, CircuitConstructionError>
     where
         Self: Sized,
@@ -225,16 +249,12 @@ pub trait CircuitNode: CircuitNodePrivate {
         self.init_info().unwrap() // we could avoid recomputing some stuff if we wanted
     }
 
-    fn self_flops(&self) -> BigUint {
-        BigUint::from(0usize)
-    }
-
     fn max_non_input_size(&self) -> BigUint {
         self.children()
             .map(|x| x.info().max_non_input_size.clone())
             .chain(std::iter::once(self.info().numel()))
             .max()
-            .unwrap()
+            .unwrap_or(0usize.into())
     }
 
     fn c(self) -> Circuit
@@ -251,11 +271,19 @@ pub trait CircuitNode: CircuitNodePrivate {
         CircuitRc(Rc::new(self.c()))
     }
 
-    fn eval_tensors(
-        &self,
-        tensors: &[Tensor],
-        device_dtype: &TorchDeviceDtype,
-    ) -> Result<Tensor, TensorEvalError>;
+    fn compiler_repr(&self) -> String
+    where
+        Self: Into<Circuit> + Clone,
+    {
+        repr_circuit_deep_compiler(&self.clone().c())
+    }
+
+    fn compiler_print(&self)
+    where
+        Self: Into<Circuit> + Clone,
+    {
+        println!("{}", self.compiler_repr())
+    }
 
     fn get_hash(&self) -> HashBytes {
         self.info().hash
@@ -287,7 +315,7 @@ macro_rules! circuit_node_eq_ord {
     ($type_name:ty) => {
         impl ::std::cmp::PartialEq for $type_name {
             fn eq(&self, other: &Self) -> bool {
-                use $crate::circuit::CircuitNode;
+                use $crate::circuit::prelude::*;
                 self.info().hash == other.info().hash
             }
         }
@@ -296,7 +324,7 @@ macro_rules! circuit_node_eq_ord {
 
         impl ::std::cmp::Ord for $type_name {
             fn cmp(&self, other: &Self) -> ::std::cmp::Ordering {
-                use $crate::circuit::CircuitNode;
+                use $crate::circuit::prelude::*;
                 // name and then
                 (self.name(), self.info().hash).cmp(&(other.name(), other.info().hash))
             }
@@ -590,6 +618,7 @@ impl<T: CircuitNodeUnion> CircuitNodePrivate for T {
     }
 }
 
+// UPDATE ME WHEN YOU CHANGE CircuitNode Trait!!!
 impl<T: CircuitNodeUnion> CircuitNode for T {
     fn info(&self) -> &CachedCircuitInfo {
         self.as_trait_obj().info()
@@ -599,10 +628,6 @@ impl<T: CircuitNodeUnion> CircuitNode for T {
         self.as_trait_obj().name()
     }
 
-    fn node_type_uuid(&self) -> [u8; 16] {
-        self.as_trait_obj().node_type_uuid()
-    }
-
     fn compute_shape(&self) -> Shape {
         self.as_trait_obj().compute_shape()
     }
@@ -612,10 +637,6 @@ impl<T: CircuitNodeUnion> CircuitNode for T {
         hasher
     }
 
-    fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
-        self.as_trait_obj().child_axis_map()
-    }
-
     fn compute_is_constant(&self) -> bool {
         self.as_trait_obj().compute_is_constant()
     }
@@ -628,6 +649,14 @@ impl<T: CircuitNodeUnion> CircuitNode for T {
         self.as_trait_obj().compute_can_be_sampled()
     }
 
+    fn device_dtype_extra<'a>(&'a self) -> Box<dyn Iterator<Item = TorchDeviceDtypeOp> + 'a> {
+        self.as_trait_obj().device_dtype_extra()
+    }
+
+    fn child_axis_map(&self) -> Vec<Vec<Option<usize>>> {
+        self.as_trait_obj().child_axis_map()
+    }
+
     fn children<'a>(&'a self) -> Box<dyn Iterator<Item = CircuitRc> + 'a> {
         self.as_trait_obj().children()
     }
@@ -640,6 +669,10 @@ impl<T: CircuitNodeUnion> CircuitNode for T {
         self.map_children_enumerate_impl(f)
     }
 
+    fn node_type_uuid(&self) -> [u8; 16] {
+        self.as_trait_obj().node_type_uuid()
+    }
+
     fn self_flops(&self) -> BigUint {
         self.as_trait_obj().self_flops()
     }
@@ -653,16 +686,6 @@ impl<T: CircuitNodeUnion> CircuitNode for T {
     }
 }
 
-impl Circuit {
-    pub fn compiler_repr(&self) -> String {
-        repr_circuit_deep_compiler(self)
-    }
-
-    pub fn compiler_print(&self) {
-        println!("{}", self.compiler_repr())
-    }
-}
-
 pub type HashBytes = [u8; 32];
 #[derive(Clone, Default)]
 pub struct CachedCircuitInfo {
@@ -1292,3 +1315,10 @@ impl PyCircuitBase {
 pub fn get_compatible_dtype(circ: &Circuit) -> TorchDeviceDtype {
     circ.info().device_dtype.clone().unwrap_or_defaults()
 }
+
+pub mod prelude {
+    pub use super::{
+        Circuit, CircuitConstructionError, CircuitNode, CircuitNodeAutoName, CircuitNodeUnion,
+        CircuitRc,
+    };
+}
