@@ -1,12 +1,9 @@
 use std::{collections::HashMap, iter::zip};
 
+use crate::cached_lambda;
 use crate::pyo3_prelude::*;
-use crate::{
-    cached_lambda,
-    circuit::{Circuit, HashBytes},
-};
 
-use super::{CircuitNode, CircuitRc, NamedAxes};
+use super::{prelude::*, HashBytes, NamedAxes};
 
 pub fn set_named_axes<T: CircuitNode + Clone>(node: &T, named_axes: NamedAxes) -> T {
     assert!(named_axes.len() == node.info().rank());
