@@ -2,10 +2,11 @@ use std::collections::HashMap;
 
 use super::{
     circuit_utils::{count_nodes, toposort_circuit, total_flops},
+    prelude::*,
     repr::ReprWrapper,
-    Circuit, CircuitNode, CircuitNodeUnion, CircuitRc, HashBytes,
+    repr::RustRepr,
+    HashBytes,
 };
-use crate::circuit::repr::RustRepr;
 use num_bigint::BigUint;
 use pyo3::pyfunction;
 
