@@ -1,7 +1,7 @@
 use uuid::Uuid;
 
 use super::{OpSize, RearrangeSpec, TensorAxisIndex, TensorIndex};
-use crate::tensor_util::Shape;
+use crate::{all_imports::TorchDeviceDtypeOp, tensor_util::Shape};
 
 pub trait RustRepr {
     /// Serialize objects to Rust expression source code
@@ -71,6 +71,20 @@ impl RustRepr for RearrangeSpec {
     }
 }
 
+impl RustRepr for TorchDeviceDtypeOp {
+    fn repr(&self) -> String {
+        if let (None, None) = (&self.device, &self.dtype) {
+            "TorchDeviceDtypeOp::default()".to_owned()
+        } else {
+            format!(
+                "TorchDeviceDtypeOp {{ device: {}, dtype: {} }}",
+                self.device.repr(),
+                self.dtype.repr(),
+            )
+        }
+    }
+}
+
 pub struct ReprWrapper(pub String);
 
 impl RustRepr for ReprWrapper {
@@ -80,14 +94,15 @@ impl RustRepr for ReprWrapper {
 }
 
 impl TensorAxisIndex {
-    pub fn repr(&self, bound: usize) -> String {
+    pub fn repr(&self, bound: usize, device_dtype: &TorchDeviceDtypeOp) -> String {
         match self {
             TensorAxisIndex::Single(i) => format!("TensorAxisIndex::Single({})", i),
             TensorAxisIndex::Tensor(t) => {
                 format!(
-                    "TensorAxisIndex::new_tensor_randint_seeded({},{},{})",
+                    "TensorAxisIndex::new_tensor_randint_seeded({}, {}, {}, {})",
                     t.shape()[0].repr(),
                     bound.repr(),
+                    device_dtype.repr(),
                     t.hash_usize().unwrap(),
                 )
             }
@@ -101,14 +116,14 @@ impl TensorAxisIndex {
 }
 
 impl TensorIndex {
-    pub fn repr(&self, shape: Shape) -> String {
+    pub fn repr(&self, shape: Shape, device_dtype: &TorchDeviceDtypeOp) -> String {
         // shape: shape of the tensor we're indexing into
         format!(
             "TensorIndex ( vec![{}] )",
             self.0
                 .iter()
                 .enumerate()
-                .map(|(i, tensor_axis_index)| tensor_axis_index.repr(shape[i]))
+                .map(|(i, tensor_axis_index)| tensor_axis_index.repr(shape[i], device_dtype))
                 .collect::<Vec<String>>()
                 .join(", ")
         )
