@@ -5,8 +5,8 @@ use std::{
 
 use super::{
     algebraic_rewrite::{get_removable_axes, remove_axes},
-    Add, Circuit, CircuitNode, CircuitRc, Concat, Einsum, Index, Rearrange, ScalarConstant,
-    Scatter,
+    prelude::*,
+    Add, Concat, Einsum, Index, Rearrange, ScalarConstant, Scatter,
 };
 use crate::pyo3_prelude::*;
 use crate::{
