@@ -14,10 +14,10 @@ use super::{
     circuit_optimizer::OptimizationSettings,
     circuit_utils::{hash_to_node, toposort_circuit},
     get_compatible_dtype,
+    prelude::*,
     print::oom_fmt,
     scheduling_z3::schedule_dag_strategy_ints,
-    visit_circuit, ArrayConstant, Circuit, CircuitNode, CircuitRc, HashBytes, ScalarConstant,
-    Symbol,
+    visit_circuit, ArrayConstant, HashBytes, ScalarConstant, Symbol,
 };
 use crate::pyo3_prelude::*;
 use crate::util::filter_to_idx;
