@@ -64,13 +64,11 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         deep_map_preorder_py, deep_map_py, filter_nodes_py, path_get, replace_nodes_py,
         update_nodes_py, update_path_py,
     };
-    use circuit::circuit_optimizer::{
-        optimize_and_evaluate, optimize_and_evaluate_many, optimize_circuit,
-    };
+    use circuit::circuit_optimizer::{optimize_and_evaluate, optimize_and_evaluate_many};
     use circuit::circuit_utils::{
         cast_circuit, count_nodes, toposort_circuit, total_arrayconstant_size, total_flops,
     };
-    use circuit::compiler_heuristics::{deep_maybe_distribute_py, maybe_distribute};
+    use circuit::compiler_heuristics::deep_maybe_distribute_py;
     use circuit::compiler_strip::strip_names;
     use circuit::concat_rewrite::{
         add_pull_concat, concat_drop_size_zero, concat_fuse, einsum_pull_concat,
