@@ -61,7 +61,7 @@ lazy_static! {
     static ref PYCIRCUIT_MODULE: Py<PyModule> = Python::with_gil(|py| {
         PyModule::from_code(
             py,
-            "from interp.circuit import computational_node, constant; import interp.circuit.interop_rust as interop; import einops; import interp.circuit.circuit_compiler.util as circ_compiler_util",
+            "from interp.circuit import computational_node, constant; import interp.circuit.interop_rust.interop_rust as interop; import einops; import interp.circuit.circuit_compiler.util as circ_compiler_util",
             "py_circuit_interop.py",
             "py_circuit_interop",
         )
