@@ -6,6 +6,7 @@ use crate::{
     pycall,
     rearrange_spec::RearrangeSpec,
 };
+use pyo3::types::IntoPyDict;
 use pyo3::{
     exceptions, ffi, pyclass, pymethods,
     types::{PySlice, PyTuple},
@@ -309,20 +310,32 @@ impl TensorAxisIndex {
         false
     }
 
-    pub fn new_tensor_randint_seeded(length: usize, base_length: usize, seed: usize) -> Self {
+    pub fn new_tensor_randint_seeded(
+        length: usize,
+        base_length: usize,
+        device_dtype: TorchDeviceDtypeOp,
+        seed: usize,
+    ) -> Self {
         TensorAxisIndex::Tensor(pyo3::Python::with_gil(|py| {
             PY_TORCH
                 .getattr(py, "manual_seed")
                 .unwrap()
                 .call(py, (seed,), None)
                 .unwrap();
+            let mut kwargs = HashMap::new();
+            if let Some(dtype) = device_dtype.dtype {
+                kwargs.insert("dtype", PY_TORCH.getattr(py, dtype).unwrap());
+            }
+            if let Some(device) = device_dtype.device {
+                kwargs.insert("device", device.into_py(py));
+            }
             PY_TORCH
                 .getattr(py, "randint")
                 .unwrap()
                 .call(
                     py,
                     (0, base_length, pyo3::types::PyTuple::new(py, vec![length])),
-                    None,
+                    Some(kwargs.into_py_dict(py)),
                 )
                 .unwrap()
                 .extract(py)
@@ -551,6 +564,15 @@ pub struct TorchDeviceDtypeOp {
     pub dtype: Option<String>,
 }
 
+#[pymethods]
+impl TorchDeviceDtypeOp {
+    #[staticmethod]
+    #[pyo3(name = "default")]
+    fn default_py() -> Self {
+        Self::default()
+    }
+}
+
 #[pyclass]
 #[derive(Debug, Clone, PartialEq, Eq, PyClassDeriv)]
 pub struct TorchDeviceDtype {
